@@ -61,6 +61,16 @@
 #include "ipv6.h"
 #include "ipv6nd.h"
 
+#ifndef ARP_MOD_NAME
+#  define ARP_MOD_NAME        "arp"
+#endif
+
+#ifndef RT_ROUNDUP
+#define RT_ROUNDUP(a)							      \
+	((a) > 0 ? (1 + (((a) - 1) | (sizeof(long) - 1))) : sizeof(long))
+#define RT_ADVANCE(x, n) (x += RT_ROUNDUP(salen(n)))
+#endif
+
 #define COPYOUT(sin, sa) do {						      \
 	if ((sa) && ((sa)->sa_family == AF_INET))			      \
 		(sin) = ((const struct sockaddr_in *)(const void *)	      \
@@ -88,11 +98,16 @@ struct dl_if {
 	uint8_t			broadcast[DLPI_PHYSADDR_MAX];
 };
 TAILQ_HEAD(dl_if_head, dl_if);
+#endif
 
 struct priv {
+#ifdef INET
 	struct dl_if_head dl_ifs;
-};
 #endif
+#ifdef INET6
+	int pf_inet6_fd;
+#endif
+};
 
 int
 if_init(__unused struct interface *ifp)
@@ -111,15 +126,20 @@ if_conf(__unused struct interface *ifp)
 int
 if_opensockets_os(struct dhcpcd_ctx *ctx)
 {
-#ifdef INET
 	struct priv		*priv;
 
 	if ((priv = malloc(sizeof(*priv))) == NULL)
 		return -1;
 	ctx->priv = priv;
+#ifdef INET
 	TAILQ_INIT(&priv->dl_ifs);
-#else
-	ctx->priv = NULL;
+#endif
+
+#ifdef INET6
+	priv->pf_inet6_fd = xsocket(PF_INET6, SOCK_DGRAM | SOCK_CLOEXEC, 0);
+	/* Don't return an error so we at least work on kernels witout INET6
+	 * even though we expect INET6 support.
+	 * We will fail noisily elsewhere anyway. */
 #endif
 
 	ctx->link_fd = socket(PF_ROUTE,
@@ -134,6 +154,13 @@ if_opensockets_os(struct dhcpcd_ctx *ctx)
 void
 if_closesockets_os(struct dhcpcd_ctx *ctx)
 {
+#ifdef INET6
+	struct priv		*priv;
+
+	priv = (struct priv *)ctx->priv;
+	if (priv->pf_inet6_fd != -1)
+		close(priv->pf_inet6_fd);
+#endif
 
 	/* each interface should have closed itself */
 	free(ctx->priv);
@@ -156,13 +183,12 @@ if_vimaster(__unused const struct dhcpcd_ctx *ctx, __unused const char *ifname)
 }
 
 int
-if_machinearch(char *str, size_t len)
+if_machinearch(__unused char *str, __unused size_t len)
 {
 
-	UNUSED(str);
-	UNUSED(len);
-	errno = ENOTSUP;
-	return -1;
+	/* There is no extra data really.
+	 * isainfo -v does return amd64, but also i386. */
+	return 0;
 }
 
 struct linkwalk {
@@ -298,30 +324,43 @@ if_getifaddrs(struct ifaddrs **ifap)
 	return 0;
 }
 
+static int
+salen(const struct sockaddr *sa)
+{
+
+	switch (sa->sa_family) {
+	case AF_LINK:
+		return sizeof(struct sockaddr_dl);
+	case AF_INET:
+		return sizeof(struct sockaddr_in);
+	case AF_INET6:
+		return sizeof(struct sockaddr_in6);
+	default:
+		return sizeof(struct sockaddr);
+	}
+}
+
+static void
+if_linkaddr(struct sockaddr_dl *sdl, const struct interface *ifp)
+{
+
+	memset(sdl, 0, sizeof(*sdl));
+	sdl->sdl_family = AF_LINK;
+	sdl->sdl_nlen = sdl->sdl_alen = sdl->sdl_slen = 0;
+	sdl->sdl_index = (unsigned short)ifp->index;
+}
+
 static int
 get_addrs(int type, const void *data, const struct sockaddr **sa)
 {
+	const char *cp;
 	int i;
-	const char *p;
 
-	p = data;
+	cp = data;
 	for (i = 0; i < RTAX_MAX; i++) {
 		if (type & (1 << i)) {
-			sa[i] = (const struct sockaddr *)p;
-			switch (sa[i]->sa_family) {
-			case AF_LINK:
-				p += sizeof(struct sockaddr_dl);
-				break;
-			case AF_INET:
-				p += sizeof(struct sockaddr_in);
-				break;
-			case AF_INET6:
-				p += sizeof(struct sockaddr_in6);
-				break;
-			default:
-				errno = EINVAL;
-				return -1;
-			}
+			sa[i] = (const struct sockaddr *)cp;
+			RT_ADVANCE(cp, sa[i]);
 		} else
 			sa[i] = NULL;
 	}
@@ -407,6 +446,26 @@ if_findsa(struct dhcpcd_ctx *ctx, const struct sockaddr *sa)
 }
 
 #ifdef INET
+static void
+if_finishrt(struct rt *rt)
+{
+
+	/* Solaris has a subnet route with the gateway
+	 * of the owning address.
+	 * dhcpcd has a blank gateway here to indicate a
+	 * subnet route. */
+	if (rt->gate.s_addr != ntohl(INADDR_ANY) &&
+	    ipv4_iffindaddr(UNCONST(rt->iface), &rt->gate, NULL))
+		rt->gate.s_addr = ntohl(INADDR_ANY);
+
+	/* Solaris likes to set route MTU to match
+	 * interface MTU when adding routes.
+	 * This confuses dhcpcd as it expects MTU to be 0
+	 * when no explicit MTU has been set. */
+	if (rt->mtu == (unsigned int)if_getmtu(rt->iface))
+		rt->mtu = 0;
+}
+
 static int
 if_copyrt(struct dhcpcd_ctx *ctx, struct rt *rt, const struct rt_msghdr *rtm)
 {
@@ -427,7 +486,7 @@ if_copyrt(struct dhcpcd_ctx *ctx, struct rt *rt, const struct rt_msghdr *rtm)
 	else
 		rt->mask.s_addr = INADDR_BROADCAST;
 	COPYOUT(rt->gate, rti_info[RTAX_GATEWAY]);
-	COPYOUT(rt->src, rti_info[RTAX_IFA]);
+	COPYOUT(rt->src, rti_info[RTAX_SRC]);
 	rt->mtu = (unsigned int)rtm->rtm_rmx.rmx_mtu;
 
 	if (rtm->rtm_index)
@@ -452,11 +511,34 @@ if_copyrt(struct dhcpcd_ctx *ctx, struct rt *rt, const struct rt_msghdr *rtm)
 		errno = ESRCH;
 		return -1;
 	}
+
+	if_finishrt(rt);
+
 	return 0;
 }
 #endif
 
 #ifdef INET6
+static void
+if_finishrt6(struct rt6 *rt)
+{
+
+	/* Solaris has a subnet route with the gateway
+	 * of the owning address.
+	 * dhcpcd has a blank gateway here to indicate a
+	 * subnet route. */
+	if (!IN6_IS_ADDR_UNSPECIFIED(&rt->gate) &&
+	    ipv6_iffindaddr(UNCONST(rt->iface), &rt->gate, 0))
+		rt->gate = in6addr_any;
+
+	/* Solarais likes to set route MTU to match
+	 * interface MTU when adding routes.
+	 * This confuses dhcpcd as it expects MTU to be 0
+	 * when no explicit MTU has been set. */
+	if (rt->mtu == (unsigned int)if_getmtu(rt->iface))
+		rt->mtu = 0;
+}
+
 static int
 if_copyrt6(struct dhcpcd_ctx *ctx, struct rt6 *rt, const struct rt_msghdr *rtm)
 {
@@ -501,6 +583,9 @@ if_copyrt6(struct dhcpcd_ctx *ctx, struct rt6 *rt, const struct rt_msghdr *rtm)
 		errno = ESRCH;
 		return -1;
 	}
+
+	if_finishrt6(rt);
+
 	return 0;
 }
 #endif
@@ -613,20 +698,14 @@ if_ifa(struct dhcpcd_ctx *ctx, const struct ifa_msghdr *ifam)
 	case AF_INET:
 	{
 		struct in_addr addr, mask, bcast;
-		int flags;
 
 		COPYOUT(addr, rti_info[RTAX_IFA]);
 		COPYOUT(mask, rti_info[RTAX_NETMASK]);
 		COPYOUT(bcast, rti_info[RTAX_BRD]);
-		if (ifam->ifam_type == RTM_NEWADDR) {
-			if ((flags = if_addrflags(&addr, ifp)) == -1)
-				break;
-		} else
-			flags = 0;
 		ipv4_handleifa(ctx,
 		    ifam->ifam_type == RTM_CHGADDR ?
 		    RTM_NEWADDR : ifam->ifam_type,
-		    NULL, ifp->name, &addr, &mask, &bcast, flags);
+		    NULL, ifp->name, &addr, &mask, &bcast);
 		break;
 	}
 #endif
@@ -635,21 +714,15 @@ if_ifa(struct dhcpcd_ctx *ctx, const struct ifa_msghdr *ifam)
 	{
 		struct in6_addr addr6, mask6;
 		const struct sockaddr_in6 *sin6;
-		int flags;
 
 		sin6 = (const void *)rti_info[RTAX_IFA];
 		addr6 = sin6->sin6_addr;
 		sin6 = (const void *)rti_info[RTAX_NETMASK];
 		mask6 = sin6->sin6_addr;
-		if (ifam->ifam_type == RTM_NEWADDR) {
-			if ((flags = if_addrflags6(&addr6, ifp)) == -1)
-				break;
-		} else
-			flags = 0;
 		ipv6_handleifa(ctx,
 		    ifam->ifam_type == RTM_CHGADDR ?
 		    RTM_NEWADDR : ifam->ifam_type,
-		    NULL, ifp->name, &addr6, ipv6_prefixlen(&mask6), flags);
+		    NULL, ifp->name, &addr6, ipv6_prefixlen(&mask6));
 		break;
 	}
 #endif
@@ -827,6 +900,258 @@ out:
 	return retval;
 }
 
+static int
+if_addaddr(int fd, const char *ifname,
+    struct sockaddr_storage *addr, struct sockaddr_storage *mask)
+{
+	struct lifreq		lifr;
+
+	memset(&lifr, 0, sizeof(lifr));
+	strlcpy(lifr.lifr_name, ifname, sizeof(lifr.lifr_name));
+
+	/* First assign the netmask. */
+	lifr.lifr_addr = *mask;
+	if (ioctl(fd, SIOCSLIFNETMASK, &lifr) == -1)
+		return -1;
+
+	/* Then assign the address. */
+	lifr.lifr_addr = *addr;
+	if (ioctl(fd, SIOCSLIFADDR, &lifr) == -1)
+		return -1;
+
+	/* Now bring it up. */
+	if (ioctl(fd, SIOCGLIFFLAGS, &lifr) == -1)
+		return -1;
+	if (!(lifr.lifr_flags & IFF_UP)) {
+		lifr.lifr_flags |= IFF_UP;
+		if (ioctl(fd, SIOCSLIFFLAGS, &lifr) == -1)
+			return -1;
+	}
+	return 0;
+}
+
+static int
+if_plumblif(int cmd, const struct dhcpcd_ctx *ctx, int af, const char *ifname)
+{
+	struct lifreq		lifr;
+	int			s;
+
+	memset(&lifr, 0, sizeof(lifr));
+	strlcpy(lifr.lifr_name, ifname, sizeof(lifr.lifr_name));
+	lifr.lifr_addr.ss_family = af;
+	if (af == AF_INET)
+		s = ctx->pf_inet_fd;
+	else {
+		struct priv	*priv;
+
+		priv = (struct priv *)ctx->priv;
+		s = priv->pf_inet6_fd;
+	}
+	return ioctl(s,
+	    cmd == RTM_NEWADDR ? SIOCLIFADDIF : SIOCLIFREMOVEIF,
+	    &lifr) == -1 && errno != EEXIST ? -1 : 0;
+}
+
+static int
+if_plumbif(const struct dhcpcd_ctx *ctx, int af, const char *ifname)
+{
+	dlpi_handle_t		dh;
+	int			fd, af_fd, mux_fd, retval;
+	struct lifreq		lifr;
+	const char		*udp_dev;
+
+	memset(&lifr, 0, sizeof(lifr));
+	switch (af) {
+	case AF_INET:
+		lifr.lifr_flags = IFF_IPV4;
+		af_fd = ctx->pf_inet_fd;
+		udp_dev = UDP_DEV_NAME;
+		break;
+	case AF_INET6:
+	{
+		struct priv *priv;
+
+		/* We will take care of setting the link local address. */
+		lifr.lifr_flags = IFF_IPV6 | IFF_NOLINKLOCAL;
+		priv = (struct priv *)ctx->priv;
+		af_fd = priv->pf_inet6_fd;
+		udp_dev = UDP6_DEV_NAME;
+		break;
+	}
+	default:
+		errno = EPROTONOSUPPORT;
+		return -1;
+	}
+
+	if (dlpi_open(ifname, &dh, DLPI_NOATTACH) != DLPI_SUCCESS) {
+		errno = EINVAL;
+		return -1;
+	}
+
+	fd = dlpi_fd(dh);
+	retval = -1;
+	mux_fd = -1;
+	if (ioctl(fd, I_PUSH, IP_MOD_NAME) == -1)
+		goto out;
+	strlcpy(lifr.lifr_name, ifname, sizeof(lifr.lifr_name));
+	if (ioctl(fd, SIOCSLIFNAME, &lifr) == -1)
+		goto out;
+
+	/* Get full flags. */
+	if (ioctl(af_fd, SIOCGLIFFLAGS, &lifr) == -1)
+		goto out;
+
+	/* Open UDP as a multiplexor to PLINK the interface stream.
+	 * UDP is used because STREAMS will not let you PLINK a driver
+	 * under itself and IP is generally  at the bottom of the stream. */
+	if ((mux_fd = open(udp_dev, O_RDWR)) == -1)
+		goto out;
+	/* POP off all undesired modules. */
+	while (ioctl(mux_fd, I_POP, 0) != -1)
+		;
+	if (errno != EINVAL)
+		goto out;
+
+	if (lifr.lifr_flags & IFF_IPV4 && !(lifr.lifr_flags & IFF_NOARP)) {
+		if (ioctl(mux_fd, I_PUSH, ARP_MOD_NAME) == -1)
+			goto out;
+	}
+
+	/* PLINK the interface stream so it persists. */
+	if (ioctl(mux_fd, I_PLINK, fd) == -1)
+		goto out;
+
+	retval = 0;
+
+out:
+	dlpi_close(dh);
+	if (mux_fd != -1)
+		close(mux_fd);
+	return retval;
+}
+
+static int
+if_unplumbif(const struct dhcpcd_ctx *ctx, int af, const char *ifname)
+{
+	struct sockaddr_storage addr, mask;
+
+	/* For the time being, don't unplumb the interface, just
+	 * set the address to zero. */
+	memset(&addr, 0, sizeof(addr));
+	addr.ss_family = af;
+	memset(&mask, 0, sizeof(mask));
+	mask.ss_family = af;
+	return if_addaddr(ctx->pf_inet_fd, ifname , &addr, &mask);
+}
+
+static int
+if_plumb(int cmd, const struct dhcpcd_ctx *ctx, int af, const char *ifname)
+{
+	struct if_spec		spec;
+
+	if (if_nametospec(ifname, &spec) == -1)
+		return -1;
+	if (spec.lun != -1)
+		return if_plumblif(cmd, ctx, af, ifname);
+	if (cmd == RTM_NEWADDR)
+		return if_plumbif(ctx, af, ifname);
+	else
+		return if_unplumbif(ctx, af, ifname);
+}
+
+static int
+if_rtmsg(unsigned char cmd, const struct interface *ifp,
+    int addrs, int flags,
+    const struct sockaddr *dst, const struct sockaddr *mask,
+    const struct sockaddr *gate, const struct sockaddr *src,
+    uint32_t mtu)
+{
+	struct rtm
+	{
+		struct rt_msghdr hdr;
+		char buffer[sizeof(struct sockaddr_storage) * RTAX_MAX];
+	} rtm;
+	char *bp = rtm.buffer;
+	size_t l;
+
+	/* WARNING: Solaris will not allow you to delete RTF_KERNEL routes.
+	 * This includes subnet/prefix routes. */
+
+	if ((cmd == RTM_ADD || cmd == RTM_DELETE || cmd == RTM_CHANGE) &&
+	    ifp->ctx->options & DHCPCD_DAEMONISE &&
+	    !(ifp->ctx->options & DHCPCD_DAEMONISED))
+		ifp->ctx->options |= DHCPCD_RTM_PPID;
+
+#define ADDSA(sa) {							      \
+		l = RT_ROUNDUP(salen((sa)));				      \
+		memcpy(bp, (sa), l);					      \
+		bp += l;						      \
+	}
+
+	memset(&rtm, 0, sizeof(rtm));
+	rtm.hdr.rtm_version = RTM_VERSION;
+	rtm.hdr.rtm_type = cmd;
+	rtm.hdr.rtm_seq = ++ifp->ctx->seq;
+	rtm.hdr.rtm_flags = flags;
+	rtm.hdr.rtm_addrs = RTA_DST | RTA_GATEWAY | addrs;
+
+	if (cmd == RTM_ADD || cmd == RTM_CHANGE) {
+		rtm.hdr.rtm_flags |= RTF_UP;
+		if (!(rtm.hdr.rtm_flags & RTF_REJECT) ||
+		    !(rtm.hdr.rtm_flags & RTF_GATEWAY))
+			rtm.hdr.rtm_addrs |= RTA_IFP;
+		if (mtu != 0) {
+			rtm.hdr.rtm_inits |= RTV_MTU;
+			rtm.hdr.rtm_rmx.rmx_mtu = mtu;
+		}
+	}
+
+	ADDSA(dst);
+	ADDSA(gate);
+	if (rtm.hdr.rtm_addrs & RTA_NETMASK)
+		ADDSA(mask);
+
+	if (rtm.hdr.rtm_addrs & RTA_IFP) {
+		struct sockaddr_dl sdl;
+
+		if_linkaddr(&sdl, ifp);
+		ADDSA((struct sockaddr *)&sdl);
+	}
+
+/* This no workie :/ */
+#if 0
+	/* route(1M) says RTA_IFA is accepted but ignored
+	 * it's unclear how RTA_SRC is different. */
+	if (rtm.hdr.rtm_addrs & RTA_IFA) {
+		rtm.hdr.rtm_addrs &= ~RTA_IFA;
+		rtm.hdr.rtm_addrs |= RTA_SRC;
+	}
+	if (rtm.hdr.rtm_addrs & RTA_SRC)
+		ADDSA(src);
+#endif
+
+#undef ADDSA
+
+	rtm.hdr.rtm_msglen = (unsigned short)(bp - (char *)&rtm);
+	if (write(ifp->ctx->link_fd, &rtm, rtm.hdr.rtm_msglen) == -1)
+		return -1;
+	ifp->ctx->sseq = ifp->ctx->seq;
+	return 0;
+}
+
+static int
+if_addrflags0(int fd, const char *ifname)
+{
+	struct lifreq		lifr;
+
+	memset(&lifr, 0, sizeof(lifr));
+	strlcpy(lifr.lifr_name, ifname, sizeof(lifr.lifr_name));
+	if (ioctl(fd, SIOCGLIFFLAGS, &lifr) == -1)
+		return -1;
+
+	return lifr.lifr_flags;
+}
+
 #ifdef INET
 const char *if_pfname = "DLPI";
 
@@ -977,10 +1302,11 @@ if_readraw(struct interface *ifp, int fd,
 	int			r;
 	size_t			mlen;
 
-	*flags = 0;
 	if ((di = if_findraw(ifp, fd)) == NULL)
 		return -1;
+	*flags = RAW_EOF; /* We only ever read one packet. */
 	mlen = len;
+	*flags = RAW_EOF; /* We only ever read one packet. */
 	r = dlpi_recv(di->dh, NULL, NULL, data, &mlen, -1, NULL);
 	return r == DLPI_SUCCESS ? (ssize_t)mlen : -1;
 }
@@ -988,31 +1314,106 @@ if_readraw(struct interface *ifp, int fd,
 int
 if_address(unsigned char cmd, const struct ipv4_addr *ia)
 {
+	struct sockaddr_storage	ss_addr, ss_mask;
+	struct sockaddr_in	*sin_addr, *sin_mask;
 
-	UNUSED(cmd);
-	UNUSED(ia);
-	errno = ENOTSUP;
-	return -1;
+	/* Either remove the alias or ensure it exists. */
+	if (if_plumb(cmd, ia->iface->ctx, AF_INET, ia->alias) == -1)
+		return -1;
+
+	if (cmd == RTM_DELADDR)
+		return 0;
+
+	if (cmd != RTM_NEWADDR) {
+		errno = EINVAL;
+		return -1;
+	}
+
+	sin_addr = (struct sockaddr_in *)&ss_addr;
+	sin_addr->sin_family = AF_INET;
+	sin_addr->sin_addr = ia->addr;
+	sin_mask = (struct sockaddr_in *)&ss_mask;
+	sin_mask->sin_family = AF_INET;
+	sin_mask->sin_addr = ia->mask;
+	return if_addaddr(ia->iface->ctx->pf_inet_fd,
+	    ia->alias, &ss_addr, &ss_mask);
 }
 
 int
-if_addrflags(const struct in_addr *addr, const struct interface *ifp)
+if_addrflags(const struct ipv4_addr *ia)
 {
+	int		flags, aflags;
 
-	UNUSED(addr);
-	UNUSED(ifp);
-	errno = ENOTSUP;
-	return -1;
+	aflags = if_addrflags0(ia->iface->ctx->pf_inet_fd, ia->alias);
+	if (aflags == -1)
+		return -1;
+	flags = 0;
+	if (aflags & IFF_DUPLICATE)
+		flags |= IN_IFF_DUPLICATED;
+	return flags;
 }
 
 int
 if_route(unsigned char cmd, const struct rt *rt)
 {
+	struct sockaddr_in	dst = {
+		.sin_family = AF_INET,
+		.sin_addr = rt->dest
+	};
+	struct sockaddr_in	mask = {
+		.sin_family = AF_INET,
+		.sin_addr = rt->mask
+	};
+	struct sockaddr_in	gate = {
+		.sin_family = AF_INET,
+		.sin_addr = rt->gate
+	};
+	struct sockaddr_in	src = {
+		.sin_family = AF_INET,
+		.sin_addr = rt->src
+	};
+	struct sockaddr_in	*g;
+	int			addrs, flags;
 
-	UNUSED(cmd);
-	UNUSED(rt);
-	errno = ENOTSUP;
-	return -1;
+	addrs = 0;
+	flags = 0;
+
+	if (cmd == RTM_ADD || cmd == RTM_CHANGE) {
+		addrs |= RTA_GATEWAY | RTA_IFP;
+		/* Subnet routes are cloning or connected if supported.
+		 * All other routes are static. */
+		if (rt->gate.s_addr != ntohl(INADDR_ANY))
+			flags |= RTF_STATIC;
+		if (rt->src.s_addr != ntohl(INADDR_ANY))
+			addrs |= RTA_IFA;
+	}
+
+	if (rt->mask.s_addr == htonl(INADDR_BROADCAST) &&
+	    rt->gate.s_addr == htonl(INADDR_ANY))
+	{
+		flags |= RTF_HOST;
+	} else if (rt->gate.s_addr == htonl(INADDR_LOOPBACK) &&
+	    rt->mask.s_addr == htonl(INADDR_BROADCAST))
+	{
+		flags |= RTF_HOST | RTF_GATEWAY;
+		/* Going via lo0 so remove the interface flags */
+		addrs &= ~(RTA_IFA | RTA_IFP);
+	} else {
+		addrs |= RTA_NETMASK;
+		if (flags & RTF_STATIC)
+			flags |= RTF_GATEWAY;
+		if (rt->mask.s_addr == htonl(INADDR_BROADCAST))
+			flags |= RTF_HOST;
+	}
+
+	if ((flags & RTF_HOST && rt->gate.s_addr == htonl(INADDR_ANY)) ||
+	    !(flags & RTF_STATIC))
+		g = &src;
+	else
+		g = &gate;
+	return if_rtmsg(cmd, rt->iface, addrs, flags,
+	    (struct sockaddr *)&dst, (struct sockaddr *)&mask,
+	    (struct sockaddr *)g, (struct sockaddr *)&src, rt->mtu);
 }
 
 static int
@@ -1051,12 +1452,13 @@ if_walkrt(struct dhcpcd_ctx *ctx, char *data, size_t len)
 
 		if_octetstr(ifname, &re->ipRouteIfIndex, sizeof(ifname));
 		rt.iface = if_find(ctx->ifaces, ifname);
-		if (rt.iface != NULL)
+		if (rt.iface != NULL) {
+			if_finishrt(&rt);
 			ipv4_handlert(ctx, RTM_ADD, &rt, 1);
-		else {
-			char destbuf[INET6_ADDRSTRLEN];
-			char gatebuf[INET6_ADDRSTRLEN];
-			const char *dest, *gate;
+		} else {
+			char		destbuf[INET6_ADDRSTRLEN];
+			char		gatebuf[INET6_ADDRSTRLEN];
+			const char	*dest, *gate;
 
 			dest = inet_ntop(AF_INET, &rt.dest,
 			    destbuf, INET6_ADDRSTRLEN);
@@ -1083,21 +1485,49 @@ if_initrt(struct dhcpcd_ctx *ctx)
 int
 if_address6(unsigned char cmd, const struct ipv6_addr *ia)
 {
+	struct sockaddr_storage	ss_addr, ss_mask;
+	struct sockaddr_in6	*sin6_addr, *sin6_mask;
+	struct priv		*priv;
+	int			r;
 
-	UNUSED(cmd);
-	UNUSED(ia);
-	errno = ENOTSUP;
-	return -1;
+	/* Either remove the alias or ensure it exists. */
+	if (if_plumb(cmd, ia->iface->ctx, AF_INET6, ia->alias) == -1)
+		return -1;
+
+	if (cmd == RTM_DELADDR)
+		return 0;
+
+	if (cmd != RTM_NEWADDR) {
+		errno = EINVAL;
+		return -1;
+	}
+
+	priv = (struct priv *)ia->iface->ctx->priv;
+	sin6_addr = (struct sockaddr_in6 *)&ss_addr;
+	sin6_addr->sin6_family = AF_INET6;
+	sin6_addr->sin6_addr = ia->addr;
+	sin6_mask = (struct sockaddr_in6 *)&ss_mask;
+	sin6_mask->sin6_family = AF_INET6;
+	ipv6_mask(&sin6_mask->sin6_addr, ia->prefix_len);
+	r = if_addaddr(priv->pf_inet6_fd,
+	    ia->alias, &ss_addr, &ss_mask);
+	if (r == -1 && errno == EEXIST)
+		return 0;
+	return r;
 }
 
 int
-if_addrflags6(const struct in6_addr *addr, const struct interface *ifp)
+if_addrflags6(const struct ipv6_addr *ia)
 {
+	struct priv		*priv;
+	int			aflags, flags;
 
-	UNUSED(addr);
-	UNUSED(ifp);
-	errno = ENOTSUP;
-	return -1;
+	priv = (struct priv *)ia->iface->ctx->priv;
+	aflags = if_addrflags0(priv->pf_inet6_fd, ia->alias);
+	flags = 0;
+	if (aflags & IFF_DUPLICATE)
+		flags |= IN6_IFF_DUPLICATED;
+	return flags;
 }
 
 int
@@ -1112,11 +1542,41 @@ if_getlifetime6(struct ipv6_addr *addr)
 int
 if_route6(unsigned char cmd, const struct rt6 *rt)
 {
+	struct sockaddr_in6	dst = {
+		.sin6_family = AF_INET6,
+		.sin6_addr = rt->dest
+	};
+	struct sockaddr_in6	mask = {
+		.sin6_family = AF_INET6,
+		.sin6_addr = rt->mask
+	};
+	struct sockaddr_in6	gate = {
+		.sin6_family = AF_INET6,
+		.sin6_addr = rt->gate
+	};
+	struct sockaddr_in6	src = {
+		.sin6_family = AF_INET6,
+		.sin6_addr = rt->src
+	};
+	struct sockaddr_in6	*g;
+	int			addrs, flags;
 
-	UNUSED(cmd);
-	UNUSED(rt);
-	errno = ENOTSUP;
-	return -1;
+	addrs = RTA_NETMASK;
+	flags = 0;
+
+	if (IN6_IS_ADDR_UNSPECIFIED(&rt->gate)) {
+		/* XXX FIXME: How to tell kernel route is subnet? */
+	} else
+		flags |= RTF_GATEWAY | RTF_STATIC;
+
+	if (IN6_IS_ADDR_UNSPECIFIED(&rt->gate))
+		g = &src;
+	else
+		g = &gate;
+
+	return if_rtmsg(cmd, rt->iface, addrs, flags,
+	    (struct sockaddr *)&dst, (struct sockaddr *)&mask,
+	    (struct sockaddr *)g, (struct sockaddr *)&src, rt->mtu);
 }
 
 static int
@@ -1154,9 +1614,10 @@ if_walkrt6(struct dhcpcd_ctx *ctx, char *data, size_t len)
 
 		if_octetstr(ifname, &re->ipv6RouteIfIndex, sizeof(ifname));
 		rt.iface = if_find(ctx->ifaces, ifname);
-		if (rt.iface != NULL)
+		if (rt.iface != NULL) {
+			if_finishrt6(&rt);
 			ipv6_handlert(ctx, RTM_ADD, &rt);
-		else {
+		} else {
 			char destbuf[INET6_ADDRSTRLEN];
 			char gatebuf[INET6_ADDRSTRLEN];
 			const char *dest, *gate;
