@@ -99,15 +99,22 @@
 #  endif
 #endif
 
-#if defined(HAVE_IN6_ADDR_GEN_MODE_NONE) || defined(ND6_IFF_AUTO_LINKLOCAL)
+#if defined(HAVE_IN6_ADDR_GEN_MODE_NONE) || defined(ND6_IFF_AUTO_LINKLOCAL) || \
+    defined(IFF_NOLINKLOCAL)
 /* If we're using a private SLAAC address on wireless,
  * don't add it until we have associated as we randomise
  * it based on the SSID. */
 #define CAN_ADD_LLADDR(ifp) \
 	(!((ifp)->options->options & DHCPCD_SLAACPRIVATE) || \
 	    (ifp)->carrier != LINK_DOWN)
-#else
+#elif __NetBSD__
+/* Earlier versions of NetBSD don't add duplicate LLADDR's if the interface
+ * is brought up and one already exists. */
 #define CAN_ADD_LLADDR(ifp) (1)
+#else
+/* We have no control over the OS adding the LLADDR, so just let it do it
+ * as we cannot force our own view on it. */
+#define CAN_ADD_LLADDR(ifp) (0)
 #endif
 
 #ifdef IPV6_MANAGETEMPADDR
@@ -559,24 +566,29 @@ ipv6_userprefix(
 void
 ipv6_checkaddrflags(void *arg)
 {
-	struct ipv6_addr *ap;
-	int ifa_flags;
+	struct ipv6_addr *ia;
+	int flags;
 
-	ap = arg;
-	ifa_flags = if_addrflags6(&ap->addr, ap->iface);
-	if (ifa_flags == -1)
-		logger(ap->iface->ctx, LOG_ERR,
-		    "%s: if_addrflags6: %m", ap->iface->name);
-	else if (!(ifa_flags & IN6_IFF_TENTATIVE)) {
-		ipv6_handleifa(ap->iface->ctx, RTM_NEWADDR,
-		    ap->iface->ctx->ifaces, ap->iface->name,
-		    &ap->addr, ap->prefix_len, ifa_flags);
+	ia = arg;
+	if ((flags = if_addrflags6(ia)) == -1) {
+		logger(ia->iface->ctx, LOG_ERR,
+		    "%s: if_addrflags6: %m", ia->iface->name);
+		return;
+	}
+
+	ia->addr_flags = flags;
+	if (!(ia->addr_flags & IN6_IFF_TENTATIVE)) {
+		/* Simulate the kernel announcing the new address. */
+		ipv6_handleifa(ia->iface->ctx, RTM_NEWADDR,
+		    ia->iface->ctx->ifaces, ia->iface->name,
+		    &ia->addr, ia->prefix_len);
 	} else {
+		/* Still tentative? Check again in a bit. */
 		struct timespec tv;
 
 		ms_to_ts(&tv, RETRANS_TIMER / 2);
-		eloop_timeout_add_tv(ap->iface->ctx->eloop, &tv,
-		    ipv6_checkaddrflags, ap);
+		eloop_timeout_add_tv(ia->iface->ctx->eloop, &tv,
+		    ipv6_checkaddrflags, ia);
 	}
 }
 #endif
@@ -610,8 +622,8 @@ ipv6_deleteaddr(struct ipv6_addr *ia)
 	}
 }
 
-int
-ipv6_addaddr(struct ipv6_addr *ap, const struct timespec *now)
+static int
+ipv6_addaddr1(struct ipv6_addr *ap, const struct timespec *now)
 {
 	struct interface *ifp;
 	struct ipv6_state *state;
@@ -742,9 +754,119 @@ ipv6_addaddr(struct ipv6_addr *ap, const struct timespec *now)
 	}
 #endif
 
+#ifdef __sun
+	/* Solaris does not announce new addresses which need DaD
+	 * so we need to take a copy and add it to our list.
+	 * Otherwise aliasing gets confused if we add another
+	 * address during DaD. */
+
+	state = IPV6_STATE(ap->iface);
+	TAILQ_FOREACH(nap, &state->addrs, next) {
+		if (IN6_ARE_ADDR_EQUAL(&nap->addr, &ap->addr))
+			break;
+	}
+	if (nap == NULL) {
+		if ((nap = malloc(sizeof(*nap))) == NULL) {
+			syslog(LOG_ERR, "%s: malloc: %m", __func__);
+			return 0; /* Well, we did add the address */
+		}
+		memcpy(nap, ap, sizeof(*nap));
+		TAILQ_INSERT_TAIL(&state->addrs, nap, next);
+	}
+#endif
+
 	return 0;
 }
 
+#ifdef ALIAS_ADDR
+/* Find the next logical aliase address we can use. */
+static int
+ipv6_aliasaddr(struct ipv6_addr *ia, struct ipv6_addr **repl)
+{
+	struct ipv6_state *state;
+	struct ipv6_addr *iap;
+	unsigned int unit;
+	char alias[IF_NAMESIZE];
+
+	if (ia->alias[0] != '\0')
+		return 0;
+	state = IPV6_STATE(ia->iface);
+
+	/* First find an existng address.
+	 * This can happen when dhcpcd restarts as ND and DHCPv6
+	 * maintain their own lists of addresses. */
+	TAILQ_FOREACH(iap, &state->addrs, next) {
+		if (iap->alias[0] != '\0' &&
+		    IN6_ARE_ADDR_EQUAL(&iap->addr, &ia->addr))
+		{
+			strlcpy(ia->alias, iap->alias, sizeof(ia->alias));
+			return 0;
+		}
+	}
+
+	unit = 0;
+find_unit:
+	if (unit == 0)
+		strlcpy(alias, ia->iface->name, sizeof(alias));
+	else
+		snprintf(alias, sizeof(alias), "%s:%u", ia->iface->name, unit);
+	TAILQ_FOREACH(iap, &state->addrs, next) {
+		if (iap->alias[0] == '\0')
+			continue;
+		if (IN6_IS_ADDR_UNSPECIFIED(&iap->addr)) {
+			/* No address assigned? Lets use it. */
+			strlcpy(ia->alias, iap->alias, sizeof(ia->alias));
+			if (repl)
+				*repl = iap;
+			return 1;
+		}
+		if (strcmp(iap->alias, alias) == 0)
+			break;
+	}
+
+	if (iap != NULL) {
+		if (unit == UINT_MAX) {
+			errno = ERANGE;
+			return -1;
+		}
+		unit++;
+		goto find_unit;
+	}
+
+	strlcpy(ia->alias, alias, sizeof(ia->alias));
+	return 0;
+}
+#endif
+
+int
+ipv6_addaddr(struct ipv6_addr *ia, const struct timespec *now)
+{
+	int r;
+#ifdef ALIAS_ADDR
+	int replaced, blank;
+	struct ipv6_addr *replaced_ia;
+
+	blank = (ia->alias[0] == '\0');
+	if ((replaced = ipv6_aliasaddr(ia, &replaced_ia)) == -1)
+		return -1;
+	if (blank)
+		logger(ia->iface->ctx, LOG_DEBUG, "%s: aliased %s",
+		    ia->alias, ia->saddr);
+#endif
+
+	if ((r = ipv6_addaddr1(ia, now)) == 0) {
+#ifdef ALIAS_ADDR
+		if (replaced) {
+			struct ipv6_state *state;
+
+			state = IPV6_STATE(ia->iface);
+			TAILQ_REMOVE(&state->addrs, replaced_ia, next);
+			ipv6_freeaddr(replaced_ia);
+		}
+#endif
+	}
+	return r;
+}
 
 int
 ipv6_findaddrmatch(const struct ipv6_addr *addr, const struct in6_addr *match,
@@ -938,12 +1060,13 @@ ipv6_getstate(struct interface *ifp)
 void
 ipv6_handleifa(struct dhcpcd_ctx *ctx,
     int cmd, struct if_head *ifs, const char *ifname,
-    const struct in6_addr *addr, uint8_t prefix_len, int flags)
+    const struct in6_addr *addr, uint8_t prefix_len)
 {
 	struct interface *ifp;
 	struct ipv6_state *state;
-	struct ipv6_addr *ap;
+	struct ipv6_addr *ia;
 	struct ll_callback *cb;
+	int flags;
 
 #if 0
 	char dbuf[INET6_ADDRSTRLEN];
@@ -951,60 +1074,55 @@ ipv6_handleifa(struct dhcpcd_ctx *ctx,
 
 	dbp = inet_ntop(AF_INET6, &addr->s6_addr,
 	    dbuf, INET6_ADDRSTRLEN);
-	logger(ctx, LOG_INFO, "%s: cmd %d addr %s flags %d",
-	    ifname, cmd, dbp, flags);
+	logger(ctx, LOG_INFO, "%s: cmd %d addr %s",
+	    ifname, cmd, dbp);
 #endif
 
 	if (ifs == NULL)
 		ifs = ctx->ifaces;
-	if (ifs == NULL) {
-		errno = ESRCH;
+	if (ifs == NULL)
 		return;
-	}
 	if ((ifp = if_find(ifs, ifname)) == NULL)
 		return;
 	if ((state = ipv6_getstate(ifp)) == NULL)
 		return;
 
-	if (!IN6_IS_ADDR_LINKLOCAL(addr)) {
-		ipv6nd_handleifa(ctx, cmd, ifname, addr, flags);
-		dhcp6_handleifa(ctx, cmd, ifname, addr, flags);
-	}
-
-	TAILQ_FOREACH(ap, &state->addrs, next) {
-		if (IN6_ARE_ADDR_EQUAL(&ap->addr, addr))
+	TAILQ_FOREACH(ia, &state->addrs, next) {
+		if (IN6_ARE_ADDR_EQUAL(&ia->addr, addr))
 			break;
 	}
 
 	switch (cmd) {
 	case RTM_DELADDR:
-		if (ap) {
-			TAILQ_REMOVE(&state->addrs, ap, next);
-			ipv6_freeaddr(ap);
+		if (ia != NULL) {
+			TAILQ_REMOVE(&state->addrs, ia, next);
+			/* We'll free it at the end of the function. */
 		}
 		break;
 	case RTM_NEWADDR:
-		if (ap == NULL) {
+		if (ia == NULL) {
 			char buf[INET6_ADDRSTRLEN];
 			const char *cbp;
 
-			ap = calloc(1, sizeof(*ap));
-			if (ap == NULL) {
+			if ((ia = calloc(1, sizeof(*ia))) == NULL) {
 				logger(ctx, LOG_ERR,
 				    "%s: calloc: %m", __func__);
 				break;
 			}
-			ap->iface = ifp;
-			ap->addr = *addr;
-			ap->prefix_len = prefix_len;
-			ipv6_makeprefix(&ap->prefix, &ap->addr,
-			    ap->prefix_len);
+#ifdef ALIAS_ADDR
+			strlcpy(ia->alias, ifname, sizeof(ia->alias));
+#endif
+			ia->iface = ifp;
+			ia->addr = *addr;
+			ia->prefix_len = prefix_len;
+			ipv6_makeprefix(&ia->prefix, &ia->addr,
+			    ia->prefix_len);
 			cbp = inet_ntop(AF_INET6, &addr->s6_addr,
 			    buf, sizeof(buf));
 			if (cbp)
-				snprintf(ap->saddr, sizeof(ap->saddr),
+				snprintf(ia->saddr, sizeof(ia->saddr),
 				    "%s/%d", cbp, prefix_len);
-			if (if_getlifetime6(ap) == -1) {
+			if (if_getlifetime6(ia) == -1) {
 				/* No support or address vanished.
 				 * Either way, just set a deprecated
 				 * infinite time lifetime and continue.
@@ -1013,8 +1131,8 @@ ipv6_handleifa(struct dhcpcd_ctx *ctx,
 				 * temporary addresses.
 				 * As we can't extend infinite, we'll
 				 * create a new temporary address. */
-				ap->prefix_pltime = 0;
-				ap->prefix_vltime =
+				ia->prefix_pltime = 0;
+				ia->prefix_vltime =
 				    ND6_INFINITE_LIFETIME;
 			}
 			/* This is a minor regression against RFC 4941
@@ -1027,33 +1145,39 @@ ipv6_handleifa(struct dhcpcd_ctx *ctx,
 			 * pretend lifetimes are infinite and always
 			 * generate a new temporary address on
 			 * restart. */
-			ap->acquired = ap->created;
-			TAILQ_INSERT_TAIL(&state->addrs,
-			    ap, next);
+			ia->acquired = ia->created;
+			TAILQ_INSERT_TAIL(&state->addrs, ia, next);
 		}
-		ap->addr_flags = flags;
+		flags = if_addrflags6(ia);
+		if (flags == -1) {
+			logger(ia->iface->ctx, LOG_ERR,
+			    "%s: %s: if_addrflags6: %m",
+			    ia->iface->name, ia->saddr);
+			return;
+		}
+		ia->addr_flags = flags;
 #ifdef IPV6_MANAGETEMPADDR
-		if (ap->addr_flags & IN6_IFF_TEMPORARY)
-			ap->flags |= IPV6_AF_TEMPORARY;
+		if (ia->addr_flags & IN6_IFF_TEMPORARY)
+			ia->flags |= IPV6_AF_TEMPORARY;
 #endif
-		if (IN6_IS_ADDR_LINKLOCAL(&ap->addr) || ap->dadcallback) {
+		if (IN6_IS_ADDR_LINKLOCAL(&ia->addr) || ia->dadcallback) {
 #ifdef IPV6_POLLADDRFLAG
-			if (ap->addr_flags & IN6_IFF_TENTATIVE) {
+			if (ia->addr_flags & IN6_IFF_TENTATIVE) {
 				struct timespec tv;
 
 				ms_to_ts(&tv, RETRANS_TIMER / 2);
 				eloop_timeout_add_tv(
-				    ap->iface->ctx->eloop,
-				    &tv, ipv6_checkaddrflags, ap);
+				    ia->iface->ctx->eloop,
+				    &tv, ipv6_checkaddrflags, ia);
 				break;
 			}
 #endif
 
-			if (ap->dadcallback)
-				ap->dadcallback(ap);
+			if (ia->dadcallback)
+				ia->dadcallback(ia);
 
-			if (IN6_IS_ADDR_LINKLOCAL(&ap->addr) &&
-			    !(ap->addr_flags & IN6_IFF_NOTUSEABLE))
+			if (IN6_IS_ADDR_LINKLOCAL(&ia->addr) &&
+			    !(ia->addr_flags & IN6_IFF_NOTUSEABLE))
 			{
 				/* Now run any callbacks.
 				 * Typically IPv6RS or DHCPv6 */
@@ -1070,6 +1194,17 @@ ipv6_handleifa(struct dhcpcd_ctx *ctx,
 		}
 		break;
 	}
+
+	if (ia != NULL) {
+		if (!IN6_IS_ADDR_LINKLOCAL(&ia->addr)) {
+			ipv6nd_handleifa(cmd, ia);
+			dhcp6_handleifa(cmd, ia);
+		}
+
+		/* Done with the ia now, so free it. */
+		if (cmd == RTM_DELADDR)
+			ipv6_freeaddr(ia);
+	}
 }
 
 int
@@ -1527,44 +1662,45 @@ ipv6_ctxfree(struct dhcpcd_ctx *ctx)
 
 int
 ipv6_handleifa_addrs(int cmd,
-    struct ipv6_addrhead *addrs, const struct in6_addr *addr, int flags)
+    struct ipv6_addrhead *addrs, const struct ipv6_addr *addr)
 {
-	struct ipv6_addr *ap, *apn;
+	struct ipv6_addr *ia, *ian;
 	uint8_t found, alldadcompleted;
 
 	alldadcompleted = 1;
 	found = 0;
-	TAILQ_FOREACH_SAFE(ap, addrs, next, apn) {
-		if (!IN6_ARE_ADDR_EQUAL(addr, &ap->addr)) {
-			if (ap->flags & IPV6_AF_ADDED &&
-			    !(ap->flags & IPV6_AF_DADCOMPLETED))
+	TAILQ_FOREACH_SAFE(ia, addrs, next, ian) {
+		if (!IN6_ARE_ADDR_EQUAL(&addr->addr, &ia->addr)) {
+			if (ia->flags & IPV6_AF_ADDED &&
+			    !(ia->flags & IPV6_AF_DADCOMPLETED))
 				alldadcompleted = 0;
 			continue;
 		}
 		switch (cmd) {
 		case RTM_DELADDR:
-			if (ap->flags & IPV6_AF_ADDED) {
-				logger(ap->iface->ctx, LOG_INFO,
+			if (ia->flags & IPV6_AF_ADDED) {
+				logger(ia->iface->ctx, LOG_INFO,
 				    "%s: deleted address %s",
-				    ap->iface->name, ap->saddr);
-				ap->flags &= ~IPV6_AF_ADDED;
+				    ia->iface->name, ia->saddr);
+				ia->flags &= ~IPV6_AF_ADDED;
 			}
 			break;
 		case RTM_NEWADDR:
 			/* Safety - ignore tentative announcements */
-			if (flags & (IN6_IFF_DETACHED |IN6_IFF_TENTATIVE))
+			if (addr->addr_flags &
+			    (IN6_IFF_DETACHED | IN6_IFF_TENTATIVE))
 				break;
-			if ((ap->flags & IPV6_AF_DADCOMPLETED) == 0) {
+			if ((ia->flags & IPV6_AF_DADCOMPLETED) == 0) {
 				found++;
-				if (flags & IN6_IFF_DUPLICATED)
-					ap->flags |= IPV6_AF_DUPLICATED;
+				if (addr->addr_flags & IN6_IFF_DUPLICATED)
+					ia->flags |= IPV6_AF_DUPLICATED;
 				else
-					ap->flags &= ~IPV6_AF_DUPLICATED;
-				if (ap->dadcallback)
-					ap->dadcallback(ap);
+					ia->flags &= ~IPV6_AF_DUPLICATED;
+				if (ia->dadcallback)
+					ia->dadcallback(ia);
 				/* We need to set this here in-case the
 				 * dadcallback function checks it */
-				ap->flags |= IPV6_AF_DADCOMPLETED;
+				ia->flags |= IPV6_AF_DADCOMPLETED;
 			}
 			break;
 		}
@@ -2211,6 +2347,7 @@ make_prefix(const struct interface *ifp, const struct ra *rap,
 		r->gate = in6addr_loopback;
 	} else
 		r->gate = in6addr_any;
+	r->src = addr->addr;
 	return r;
 }
 
@@ -2319,11 +2456,14 @@ ipv6_buildroutes(struct dhcpcd_ctx *ctx)
 
 	/* Should static take priority? */
 	ipv6_build_static_routes(ctx, &dnr);
+#ifdef HAVE_ROUTE_METRIC
+	rt = TAILQ_LAST(&dnr, rt6_head);
+#endif
 
 	/* First add reachable routers and their prefixes */
 	ipv6_build_ra_routes(ctx->ipv6, &dnr, 0);
 #ifdef HAVE_ROUTE_METRIC
-	have_default = (TAILQ_FIRST(&dnr) != NULL);
+	have_default = (rt != TAILQ_LAST(&dnr, rt6_head));
 #endif
 
 	/* We have no way of knowing if prefixes added by DHCP are reachable
@@ -2356,15 +2496,14 @@ ipv6_buildroutes(struct dhcpcd_ctx *ctx)
 		/* Is this route already in our table? */
 		if (find_route6(nrs, rt) != NULL)
 			continue;
-		//rt->src.s_addr = ifp->addr.s_addr;
 		/* Do we already manage it? */
 		if ((or = find_route6(ctx->ipv6->routes, rt))) {
 			if (or->iface != rt->iface ||
 #ifdef HAVE_ROUTE_METRIC
 			    rt->metric != or->metric ||
 #endif
-		//	    or->src.s_addr != ifp->addr.s_addr ||
 			    !IN6_ARE_ADDR_EQUAL(&or->gate, &rt->gate) ||
+			    // !IN6_ARE_ADDR_EQUAL(&or->src, &rt->src) ||
 			    or->mtu != rt->mtu)
 			{
 				if (c_route(or, rt) != 0)
